#!/bin/python
# Local homology of a simulated wireless network
#
# Copyright (c) 2015, Michael Robinson
# Distribution of unaltered copies permitted for noncommercial use only
# All other uses require express permission of the author
# This software comes with no warrantees express or implied 

import numpy as np
import matplotlib.pyplot as plt
import simplicialHomology as sh

def dist(p1,p2):
    return np.sqrt(np.sum(np.abs(p1-p2)**2))

def link_complex(nodes,thres):
    """Construct the link complex for a collection of nodes using a distance threshold"""
    return toplexify([[idx for idx,nd in enumerate(nodes) if dist(nd,node)<thres] for node in nodes])

def interference_complex(nodes,thres,receivers):
    """Construct the interference complex for a collection of nodes using a set of receivers for witnesses"""
    return toplexify([[idx for idx,nd in enumerate(nodes) if dist(rx,nd)<thres ] for rx in receivers])

def toplexify(simplices):
    """Reduce a simplicial complex to merely specification of its toplices"""
    return [spx for i,spx in enumerate(simplices) if not [sp2 for j,sp2 in enumerate(simplices) if (i!=j and set(spx).issubset(sp2))]]

def plot_complex(locations,complex,color=None):
    """Plot a complex given locations of nodes"""
    colors='cbgr'
    for idx,f in enumerate(complex):
        if color != None:
            col=colors[color[idx]]
        else:
            col='g'
        if len(f)>2:
            for face in sh.ksublists(f,3):
                x,y=zip(*[locations[i] for i in face])
                plt.fill(x,y,col,alpha=0.5)
        elif len(f)>=2:
            x,y=zip(*[locations[i] for i in f])
            plt.fill(x,y,col,alpha=0.5)

    x,y=zip(*locations)
    plt.plot(x,y,'k.',ms=4*np.pi)
    return

# Simulation parameters
ers=10    # End region size
iw=2      # Width of vulnerable connection
il=5      # Length of vulnerable connection
cr=5     # Connection radius
nodes=20  # Number of nodes to try (fewer are actually generated)
rxs=20    # Number of receivers

# Generate node locations
nodelist=[]
for i in range(nodes):
    nd=np.random.rand(2)*[(2.*ers+il),ers]-[(2*ers+il)/2.,ers-ers/2.]
    if ((nd[0]<-il/2.) or (nd[0]>il/2.)) or ((nd[1]<iw/2.) and (nd[1]>-iw/2.)):
        nodelist.append(nd)

# Generate receiver locations
rxlist=[np.random.rand(2)*[(2*ers+il),ers]-[(2*ers+il)/2,ers-ers/2] for i in range(rxs)]

# Build simplicial complex
#complex=link_complex(nodelist,cr)
complex=interference_complex(nodelist,cr,rxlist)

## Display output
#plot_complex(nodelist,complex)
#plt.savefig('link_complex.png')
#plt.show()

# Walk over toplices, producing local homology
lh=np.array([1+sh.localHomology(complex,1,tplx).shape[1] for tplx in complex])
plot_complex(nodelist,complex,lh)
plt.savefig('local_betti1_0.png')
plt.show()

## Display output

#plt.show()

